/*
 * Test.c
 *
 * Created: 15.09.2012 17:05:48
 *  Author: Simon
 */ 


#include <avr32/io.h>
#include "compiler.h"
#include "gpio.h"
#include "delay.h"
#include "board.h"
#include "power_clocks_lib.h"
#include "pm_uc3c.h"
#include "scif_uc3c.h"
#include "can.h"
#include "canif.h"
#include "intc.h"
#include "conf_can.h"
#include "conf_can_example.h"

//! Local allocation for MOB buffer in HSB_RAM
#if defined (__GNUC__) && defined (__AVR32__)
volatile can_msg_t mob_ram_ch0[NB_MOB_CHANNEL] __attribute__ ((section (".hsb_ram_loc")));
#elif defined (__ICCAVR32__)
volatile __no_init can_msg_t mob_ram_ch0[NB_MOB_CHANNEL] @0xA0000000;
#endif

unsigned char mode1 = 0;
unsigned long i = 0;

void BoardInit(void);
void can_out_callback_channel0(U8 handle, U8 event);
void can_example_prepare_data(U8 mode);

int main(void)
{
	BoardInit();
	
	//Disable all interrupts.
	Disable_global_interrupt();

	// Initialize interrupt vectors.
	INTC_init_interrupts();
	
	static const gpio_map_t CAN_GPIO_MAP =
	{
		{AVR32_CANIF_RXLINE_0_0_PIN, AVR32_CANIF_RXLINE_0_0_FUNCTION},
		{AVR32_CANIF_TXLINE_0_0_PIN, AVR32_CANIF_TXLINE_0_0_FUNCTION}
	};
	
	// Assign GPIO to CAN.
	gpio_enable_module(CAN_GPIO_MAP,sizeof(CAN_GPIO_MAP) / sizeof(CAN_GPIO_MAP[0]));
	
	// Initialize channel 0
	can_init(0,((U32)&mob_ram_ch0[0]),CANIF_CHANNEL_MODE_LISTENING,can_out_callback_channel0);

	// Enable all interrupts.
	Enable_global_interrupt();	

	while(1)
	{
		gpio_tgl_gpio_pin(LED2_GPIO);
		can_example_prepare_data(0);
		delay_ms(2000);	
	}
}

void BoardInit(void)
{
	// Switch the main clock to OSC0
	scif_configure_osc_crystalmode(SCIF_OSC0, FOSC0);
	scif_enable_osc(SCIF_OSC0, OSC0_STARTUP, true);
	pm_set_mclk_source(PM_CLK_SRC_OSC0);

	// Setup the generic clock for CAN
	scif_gc_setup(AVR32_SCIF_GCLK_CANIF,SCIF_GCCTRL_OSC0,AVR32_SCIF_GC_DIV_CLOCK,2);
	// Now enable the generic clock
	scif_gc_enable(AVR32_SCIF_GCLK_CANIF);
	
	delay_init(FOSC0);											//Delay-Funktionalitt mit Systemtakt initalisieren
	
	gpio_enable_module_pin(AVR32_PIN_PA08, GPIO_DIR_OUTPUT);	//PA08 als Ausgang setzen
	gpio_enable_module_pin(AVR32_PIN_PD23, GPIO_DIR_OUTPUT);	//PD23 als Ausgang setzen
}


void can_example_prepare_data(U8 mode)
{
	// Prepare Transmission Mode
	if (mode ==0)
	{
		// Initialize channel 0
		can_init(0,((U32)&mob_ram_ch0[0]),CANIF_CHANNEL_MODE_NORMAL,can_out_callback_channel0);
		// Allocate one mob for TX
		appli_tx_msg.handle = can_mob_alloc(0);

		// Check return if no mob are available
		if (appli_tx_msg.handle==CAN_CMD_REFUSED)
		{
			//while(1);
		}

		can_tx(0,appli_tx_msg.handle,appli_tx_msg.dlc,appli_tx_msg.req_type,appli_tx_msg.can_msg);
}
	// Prepare Reception Mode
	else if (mode == 1)
	{
		can_init(0,
		((U32)&mob_ram_ch0[0]),
		CANIF_CHANNEL_MODE_NORMAL,
		can_out_callback_channel0);

		// Allocate one mob for RX
		appli_rx_msg.handle = can_mob_alloc(0);

		can_rx(0,
		appli_rx_msg.handle,
		appli_rx_msg.req_type,
		appli_rx_msg.can_msg);
	}
	// Prepare Remote Reception Mode
	else if (mode == 2)
	{
		can_init(0,
		((U32)&mob_ram_ch0[0]),
		CANIF_CHANNEL_MODE_NORMAL,
		can_out_callback_channel0);

		// Allocate one mob for RX
		appli_remote_rx_msg.handle = can_mob_alloc(0);

		can_rx(0,
		appli_remote_rx_msg.handle,
		appli_remote_rx_msg.req_type,
		appli_remote_rx_msg.can_msg);
	}

}


void can_out_callback_channel0(U8 handle, U8 event)
{	
	switch(event)
	{
		case CAN_STATUS_COMPLETED:
		{
			gpio_tgl_gpio_pin(LED0_GPIO);
			break;
		}
		case CAN_STATUS_BUSOFF:
		{
			gpio_tgl_gpio_pin(LED1_GPIO);
			break;
		}
		case CAN_STATUS_ERROR:
		{
			gpio_tgl_gpio_pin(LED1_GPIO);
			break;
		}
	}
	
	switch(mode1)
	{
		// Transmission Only
		case 0:
		appli_tx_msg.status = event;
		can_mob_free(0,handle);
		break;
		// Reception Only
		case 1:
		appli_rx_msg.can_msg->data.u64 = can_get_mob_data(0,handle).u64;
		appli_rx_msg.can_msg->id = can_get_mob_id(0,handle);
		appli_rx_msg.dlc = can_get_mob_dlc(0,handle);
		appli_rx_msg.status = event;
		can_mob_free(0,handle);
		break;
		// Remote Reception Only
		case 2:
		appli_remote_rx_msg.status = event;
		can_mob_free(0,handle);
		break;
	}
}


							